/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * IndicatorPanel.java
 *
 */

package com.exponto.jstock;

/**
 * Makes a separate JPanel for an indicator graph. The actual drawing is
 * done by the indicator class itself.
 * 
 * @author  kaare
 * @version 
 */
//import java.awt.image.VolatileImage;
import java.awt.image.BufferedImage;
import javax.swing.JPanel;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.Transparency;
import java.awt.geom.Line2D;
//import java.awt.GraphicsConfiguration;
import com.exponto.jstock.indicators.Indicator;
import org.apache.log4j.Logger;
 
public class IndicatorPanel extends JPanel{
	private Indicator indicator;
	private GraphicsParameters graphicsParameters;
	private GraphicsParameters oldGraphicsParameters;
	private BufferedImage bImage = null;
	static Logger logger = Logger.getLogger(IndicatorPanel.class);
	LogDataSeries lDS;
	private int tMargin = 10;
	private int bMargin = 10;
	private int dailyPrintout = -1;
	
    /** Creates new Panel */
    public IndicatorPanel(LogDataSeries logDS, Indicator ind) {
    	indicator = ind;
    	lDS = logDS;
		setVisible(true);
		indicator.initiateIndicator(logDS);
		//repaint();
    }
    
    public void setGraphicsParameters (GraphicsParameters gP) {
		graphicsParameters = gP;
		this.setPreferredSize(gP.getDimension());
		this.setMaximumSize(gP.getDimension());
		graphicsParameters.setBottomMargin(bMargin);
		graphicsParameters.setTopMargin(tMargin);
		//logger.debug("GraphicsParameters (dim, top, bottom margin"+ gP.getDimension().toString() + ", " + graphicsParameters.getTopMargin() + ", " + graphicsParameters.getBottomMargin());
    }

	public void updateDailyPrintout(int number) {
		dailyPrintout = number;
		repaint();
		//this.markDailyPrintout()
	}
    
	public void paint(Graphics gr) {
		Graphics2D gr2d = (Graphics2D) gr;
		//logger.debug("IndicatorPanel, real width, height: " + this.getWidth() + ", " + this.getHeight());
		super.paint(gr2d);
		if (!this.imageValid(indicator)) {
			int width =  (int) graphicsParameters.getDimension().getWidth();
			int height = (int) graphicsParameters.getDimension().getHeight();
			bImage = gr2d.getDeviceConfiguration().createCompatibleImage(width, height, Transparency.TRANSLUCENT);
			this.drawImage();
		}
//		 else {
//			logger.debug("Buffered Image is valid");
//		}
		gr2d.drawImage(bImage, 0, 0, this);
		String s = indicator.getDailyPrintOut(dailyPrintout);
		gr2d.drawString(s, graphicsParameters.getLeftMargin(), tMargin);
		if (dailyPrintout > 0) {
			this.markDailyPrintout(gr2d);
		}
		//logger.debug("Daily printout is: " + s);
	}

	private boolean imageValid(Indicator ind) {
		boolean isValid = true;
		// Initialise oldGrahicsParameters on first run (will always be equal)
		if (oldGraphicsParameters == null) {
			oldGraphicsParameters = (GraphicsParameters) graphicsParameters.clone();
			isValid = false;
			//logger.debug("Image is not valid due to: No oldGraphics" );
		} 
		if (bImage == null) {
			isValid = false;
			//logger.debug("Image is not valid due to: bImage is null" );
		}
		// Check if indicator parameters has changed
		if (ind.hasChanged()) {
			isValid = false;
			indicator.initiateIndicator(lDS);
			//logger.debug("Image is not valid due to: Indicator has changed" );
		}
		if (!graphicsParameters.isEqual(oldGraphicsParameters)) {
			isValid = false;
			//logger.debug("Image is not valid due to: Graphics has changed" );
		} 
		//logger.debug("Image is " + isValid);
		return isValid;
	}
	
	private void drawImage(){

		// Get the Graphics
		Graphics2D bImageGraphics = bImage.createGraphics();
		bImageGraphics.setBackground(Color.lightGray);
		// Paints the Image
		indicator.paintIndicator(bImageGraphics, graphicsParameters);
		// Debug code - remove
//		String [] print = indicator.printAll(100);
//		for (int i = 0; i < print.length; i++) {
//			System.out.println(print[i]);
//		}
		// end debug code
		//logger.debug("BufferedImage painted");
	}

	private void markDailyPrintout(final Graphics2D g2d){
		if (dailyPrintout >= 0) {
			Line2D line = new Line2D.Double ();
			int x =graphicsParameters.getLeftMargin() + graphicsParameters.getPixelPrDay() * dailyPrintout -1;
			double y0 = graphicsParameters.getDimension().getHeight() - bMargin;// + tMargin;
			double y1 = tMargin;
			// orange may give better visibility for the candlestick itself
			g2d.setColor(Color.gray);
			double y;
			boolean draw = true;
			// Draw dashed line
			for (y = y1; y < y0; y = y + 4) {
				if (draw) {
					line.setLine(x, y, x, y+4);    
					g2d.draw(line);
					line.setLine(x + graphicsParameters.getPixelPrDay(), y, x + graphicsParameters.getPixelPrDay(), y + 4);
					g2d.draw(line);
				}
				draw = !draw;
			}
			// Use this for solid line
//			line.setLine(x, y1, x + graphicsParameters.getPixelPrDay(), y1);
//			g2d.draw(line);
			g2d.setColor(Color.black); 
		}
	}

}
