/*
 *                                 jStock
 *                    (C) Copyright 2003, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * LogTradingData.java
 *
 * Created on 01 November 2001, 17:10
 */

package com.exponto.jstock;

import com.exponto.util.*;
//import com.exponto.util.*;

/**
 * Contains all data for a particular stock for a single day. 
 * 
 * Because Rectangle2D does not understand negative heigth it uses
 * (logx, -, -, logh) where logx is the largest value of open and close.
 *
 * @author  kaare
 * @version 0.9
 */

public class LogTradingData  implements Cloneable {
    double logHigh, logLow, logDiff, open, close, high, low, logX = 0.0;
    long volume = 0L;
	char status = Constants.unknown;
    Date day = new Date();
    /** 
     * Creates new LogChartData
     *  
     * If real data is added later, be sure to update the status field
     * 
     * */
    public LogTradingData() {
    	status = Constants.missing;
    }

	/** 
	 * Creates new LogChartData with complete dataset
	 * 
	 * */
 
	public LogTradingData(Date d, double o, double h, double l, double c, long v, char s) {
	// Constructor for parsed data in Netfonds format
		day = d.klone();            
		volume = v;
		open = o;
		high = h;
		low = l;
		close = c;
		status = s;
		computeLogValues();
	}        
    
/**
 * 
 * Creates new LogChartData with only the status field set.
 * 
 * @param s status field
 */    
	public LogTradingData(char s) {
		status = s;
	}
/**
 * Computes the logaritmic values that normally is stored with the data.
 *
 */
    
	public void computeLogValues() {
		if (this.isTraded() ) {
			double log10 =  Math.log(10);
			logHigh = Math.log(high)/log10;
			logLow = Math.log(low)/log10;
			if (open > close) { 
				logX = Math.log(open)/log10;
				logDiff = logX - Math.log(close)/log10;
			} else {
				logX = Math.log(close)/log10;
				logDiff = logX - Math.log(open)/log10;
			}
		}
	}
/**
 * Note that clone() doesn't clone the date, so don't change unless intend to change it for all others using that date. 
 * All other values are real copies so they may be changed without such consequences.
 */    
    public Object clone () {
    	if (this == null) return null;
    	LogTradingData newData = new LogTradingData();
		newData.logHigh = this.logHigh;
		newData.logLow = this.logLow;
		newData.logDiff = this.logDiff;
		newData.open = this.open;
		newData.close = this.close;
		newData.high = this.high;
		newData.low = this.low;
		newData.logX = this.logX;
		newData.volume = this.volume;
		newData.status = this.status;
		// Note that day is not cloned, only reference is copied
		newData.day = this.day;
    	return newData;
    }
    /** 
     * Check if the LogTradingData should be used in further processing.
     * If status is unknow, missing, faulty or drop data is considered unusable
     * 
     * Note that no trade is considered "normal", even if they might mess up 
     * things if ignored
     * 
     * @return boolean 	true is data can be used in further processing
     * 								false if data is not to be trusted
     */
    public boolean isUsable() {
    	boolean result = true;
    	if (this.status == Constants.missing || this.status == Constants.unknown || this.status == Constants.faulty || this.status == Constants.drop ) {
    		result = false;
    	} 
    	return result;
    }

	/** 
	 * Check if the LogTradingData is traded.
	 * If status is unknow, missing, faulty drop, or no trade false is returned
	 * 
	 * @return boolean 	true is data is traded
	 * 								false not traded or faulty data
	 */
	public boolean isTraded() {
		boolean result = true;
		if (!this.isUsable() || this.status == Constants.noTrade ) {
			result = false;
		} 
		return result;
	}
	/**
	 * @return the closing price of the current dataset
	 */
	public double getClose() {
		return close;
	}


	/**
	 * @return the date of the current dataset
	 */
	public Date getDate() {
		return day;
	}

	/**
	 * @return the status of the current dataset
	 */
	public char getStatus() {
		return status;
	}
	
	public void setStatus() {
		status = Util.validate(this);
	}

	/**
	 * @return the volume of the current dataset
	 */
	public long getVolume() {
		return volume;
	}
	
	public String toString() {
		String s;
		s = day.toDigitString() + " Close : " + close + " High:" + high + " Status: " + status;;
		return s;
	}
	
	public boolean equals(Object o) {
		boolean result = true;
		LogTradingData lTD = (LogTradingData) o;
		if (this.close != lTD.close) result = false;
		if (this.open != lTD.open) result = false;
		if (this.high != lTD.high) result = false;
		if (this.low != lTD.low) result = false;
		if (this.volume != lTD.volume) result = false;
		if (this.logHigh != lTD.logHigh) result = false;
		if (this.logLow != lTD.logLow) result = false;
		if (this.logDiff != lTD.logDiff) result = false;
		if (this.logX != lTD.logX) result = false;
		if (this.day.compareDate(lTD.day) != 0) result = false;
		return result;
	}
}
