/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/*
 * Created on 04-Mar-2004
 *
 */
package com.exponto.jstock.indicators;
import com.exponto.jstock.LogDataSeries;
import com.exponto.jstock.GraphicsParameters;
import java.awt.Graphics2D;
//import java.util.ArrayList;
//import javax.swing.JComponent;
/**
 * The IndicatorInterface defines how the rest of the applications should
 * interact with different indicators.
 * 
 * It is assumed that indicators has a valid start and stop date that is 
 * dervied from the input data to the indicator (LogDataSeries) and indicator 
 * spesific parameters (e.g. a Moving Average computed over 200 days, will 
 * not be valid for the first 200 days in a LogDataSeries.). Further more it is 
 * required that the indicator can display itself correctly on the screen, based 
 * on the size of the JPanel (or possibly the Graphics2D) and the width of 
 * candlesticks. It is also required that a indicator can give information about 
 * a particular day. 
 * 
 * Mouse and/or keyboard input is currently not part of this interface
 * 
 * Note that the daily LogDataSeries assumes that all weeks has five valid 
 * days. 
 * 
 * @author kaare
 *
 */
public interface IndicatorInterface {
	
	/**
	 * This returns the one and only IndicatorParameter used together with
	 * a specific indicator. That is even if each and every chart has a seperate
	 * instance of the indicator class, all this instances have to share the
	 * indicator parameters.  
	 * 
	 * @return the IndicatorParameter associated with the specific indicator.
	 */
	public IndicatorParameters getIndicatorParameters();
	
	/**
	 * Initiates an indicator with the 
	 * provided LogDataSeries combined with indicator spesific parameters 
	 * either provied from a properties file or the user.
	 * 
	 *  
	 * @param lDS The LogDataSeries containg values to computes the indicator
	 */
	public void initiateIndicator(LogDataSeries lDS);
	
	/**
	 * As above but with Indicatorparameters provided directly
	 * 
	 * @param lDS
	 * @param indParam
	 */
	public void initiateIndicator(LogDataSeries lDS, IndicatorParameters indParam);

	/**
	 * Information for the class that allocates screenspace wheter a particular 
	 * indicator requires a separate screen space or if it shall be painted with 
	 * the candlesticks.
	 * 
	 * @return true if this indicator shall be drawn in the same panel as the
	 * candlesticks, false if it shall be drawn in a separate panel
	 */
	public boolean drawWithCandleSticks();

	/**
	 * Check if the calling class can use data obtained from a previous
	 * call to paintIndicator or not. The intention is that the calling
	 * class may store the graphs in an image in order to speed up the
	 * drawing time (at the cost of memory)
	 * 
	 * @return true if the parameters for this indicator has changed, 
	 * false otherwise
	 */
	public boolean hasChanged();
	/**
	 * Sends the Indicator a Graphics2D and asks it to paint itself on that 
	 * Graphics.
	 * 
	 * @param g2d the Graphics to paint on
	 */
	public void paintIndicator(Graphics2D g2d, GraphicsParameters gP);

	/**
	 * Makes a String with name, parameters and value (e.g. "RSI(14): 74") 
	 * that can be used to print information about a particular day on screen. 
	 * 
	 * The Indicators them self has only indirect knowledge about dates other 
	 * than start and stop date for the indicator, so the indicator number 
	 * displayed on screen are used. 0 is the leftmost visible indicator.   
	 *  
	 * @param n the number of day (0 is the first visible day on screen 
	 * @return a String with Indicator name and value(s) that may 
	 * 				be displayed on screen
	 */
	public String getDailyPrintOut(int n);
	
	/**
	 * Generates an Array of strings that is suitable for printing to file or 
	 * debugging purposes with all data within the range. There shall be one 
	 * String per timeperiode (e.g. one per day for daily charts). A header 
	 * String may also be generated.
	 * 
	 * Note that the start and stop values shall be based on the original 
	 * LogDataSeries and not the indicator series. If the indicatorSeries
	 * has no valid data for parts of the series "not valid" may be printed. 
	 * 
	 * 
	 * @param number of values to print, starting from the latest  
	 * @return a String array with all values within the range
	 */
	public String[] printAll(int number);
	
	/**
	 * Return true if theIndicatorValuesInterface is implemented for this class.
	 * 
	 * @return true if the IndicatorValuesInterface is supporteted, false otherwise.
	 */
	public boolean supportIndicatorValuesInterface();

}

