/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 12-Jun-2004
 *
 */
package com.exponto.jstock.indicators;

/**
 * Allows other parts of the application to access parameters and values of an 
 * indicator. 
 * 
 * It's up to the classes implementing this interface to decide what parameters and 
 * values that are exposed.
 *  
 * The intended use of this interface is to present all avaliable paramters to the user,
 * so that (s)he can set up a list of rules that would trigger events like a sell or buy
 * signal.
 * 
 * Examples: 
 * 	RSI < 20 triggers buy signal
 * 	MACD < MACD.triggerline triggers sell signal  
 * 
 * @author kaare
 *
 */
public interface IndicatorValuesInterface {
	
	/**
	 * Gives the calling class a list of parameters that is available.
	 * 
	 * The parameters name shall have print quality.
	 * 
	 * @return a string array of the parameters that is available
	 */
	public String[] parameters ();
	
	/**
	 * Gives the calling class a string array with tool tips that can be presented to the
	 * user. 
	 * 
	 * It is crucial that the tool tip is in the same order as the parameter name, but null is 
	 * a valid tool tip.
	 * 
	 * The tool tips shall have print quality.
	 * @return a string array with tool tip explaing the parameters in more detail
	 */
	public String[] parametersToolTip ();
	
	/**
	 *Returns the value for the last day of the requested parameter. null shall be returned 
	 *if no value is avaliable. 
	 * 
	 * Assumes that all parameters are doubles.
	 *  
	 * @param parameter The name of the parameter
	 * @return  the last value of the requested parameter.
	 */
	public Double parameterValue(String parameter);

	/**
	 *Returns the value for the last day minus index of the requested parameter. null 
	 *shall be returned if no value is avaliable. 
	 * 
	 * The value of index has to be verified against the real length of the 
	 * indicatorSeries as the calling class has no idea about that.
	 * 
	 * Assumes that all parameters are doubles.
	 *  
	 * @param parameter The name of the parameter
	 * @param index The index
	 * @return  the value of the requested parameter for the requested day.
	 */
	public Double parameterValue(String parameter, int index);
	
}
